/*
 * The Desktop Contemplator for Nuts and Volts
 *
 * A program to display fractals and other patterns on desktop device
 * with the intent to distract and calm the mind.
 *
 * Hardware:
 *   An Arduino Uno processor board
 *   SainSmart 3.2" TFT LCD Display (5V part) 
 *
 * Concept and Implementation by: Craig A. Lindley
 * Version: 1.0
 * Last Update: 11/11/2012
 *
 * Uses the UTFT Library available from: http://henningkarlsen.com/electronics/library.php?id=52
 */

#include <setjmp.h>
#include <UTFT.h>

extern uint8_t BigFont[];

// A function pointer for executing display patterns from an array
typedef void (*pt2Function)();

// Array of pointers to pattern display functions
pt2Function patternFunctions [] = {  

  bigPixelPattern,
  burstPattern,
  offsetCirclePattern,
  juliaPattern,
  plasmaPattern,
  horizPaletteLinesPattern,
  bigPixel2Pattern,
  vertPaletteLinesPattern,
  concentricSquaresPattern,
  textPattern,
  circleOfCirclesPattern,
  concentricCirclesPattern,
  circlesPattern,
  linesPattern,
  webPattern,
  spirographPattern,
  mandelbrotPattern,
  juliaPattern,
  plasmaPattern,
};

// Determine the number of display patterns from the entries in the array
#define NUMBER_OF_PATTERNS (sizeof(patternFunctions) / sizeof(pt2Function))

// Create array of flags for display pattern selection
byte flags[NUMBER_OF_PATTERNS];

/*******************************************************************/
/***                       System Setup                          ***/
/*******************************************************************/

// Setup a jmp buffer for holding long jump state
jmp_buf env;

// Create an instance of the LCD graphics object called "lcd"
// The LCD display uses the SSD1289 driver
// The numerical values are pin values for Arduino in landscape orientation

// Uncomment out next line when Sainsmart LCD display connected directly to Arduino Uno
UTFT lcd(SSD1289, 19, 18, 17, 16, LANDSCAPE);

// Uncomment out next line when running on Sainsmart Arduino Mega 2560 + LCD shield + LCD display
// UTFT lcd(SSD1289, 38, 39, 40, 41, LANDSCAPE);

void setup() {

  // Setup serial interface for debugging
  // NOTE: Cannot use Serial and LCD at the same time
  // Serial.begin(115200);

  // Use the analog inputs to seed random number generator
  // All analog lines to input
  pinMode(A0, INPUT);
  pinMode(A1, INPUT);
  pinMode(A2, INPUT);
  pinMode(A3, INPUT);
  pinMode(A4, INPUT);
  pinMode(A5, INPUT);

  // Read each input
  int v0 = analogRead(A0);
  int v1 = analogRead(A1);
  int v2 = analogRead(A2);
  int v3 = analogRead(A3);
  int v4 = analogRead(A4);
  int v5 = analogRead(A5);

  long total = v0 + v1 + v2 + v3 + v4 + v5;

  // Seed the generator
  randomSeed(total);

  // All analog lines to output for LCD display (on Uno)
  pinMode(A0, OUTPUT);
  pinMode(A1, OUTPUT);
  pinMode(A2, OUTPUT);
  pinMode(A3, OUTPUT);
  pinMode(A4, OUTPUT);
  pinMode(A5, OUTPUT);

  // Setup the LCD display
  lcd.InitLCD();

  // Select fontfloat
  lcd.setFont(BigFont);

  // Clear flags array
  memset(flags, 0, sizeof(flags));
}

/*******************************************************************/
/***                      Main Program Loop                      ***/
/*******************************************************************/

void loop() {

  // Setup the jmp buffer so control always returns to top of this loop  
  setjmp(env);

  while (true) {
    // Select and run a display pattern until timeout occurs
    selectPatternAndRun();
  }
}

/*******************************************************************/
/***                Pattern Display Infrastructure               ***/
/*******************************************************************/

// Pattern display timeout and associated variables
#define PATTERN_DISPLAY_DURATION_SECONDS 30
#define BETWEEN_PATTERN_DELAY_SECONDS     2

// How many time infrastructure will attempt to generate a random index
#define MAX_SPINS 100

boolean timeOutEnabled;
unsigned long timeOut;

// Check for pattern timeout
// This must be called by everyt display pattern 
void checkForTimeout() {

  boolean timeOutCondition = timeOutEnabled && (millis() > timeOut);

  if (! timeOutCondition) {
    return;
  }
  timeOutEnabled = false;

  // Do the long jump back into top of main loop
  longjmp(env, 1);
}

// Randomly select a pattern to run
// Return all patterns before allowing any repeats
int selectPattern() {

  int index;
  int spins = 0;

  while (true) {
    // Pick a candidate pattern index
    index = random(NUMBER_OF_PATTERNS);

    // Check to see if this pattern was previously selected
    if (flags[index] == 0) {
      // This index was not previously used, so mark it so
      flags[index] = 1;
      return index;
    }  
    else  {
      if (++spins > MAX_SPINS) {
        memset(flags, 0, sizeof(flags));
        spins = 0;
      }
    }
  }
}

// Select a display pattern randomly and execute it
void selectPatternAndRun() {

  // First clear the screen
  lcd.clrScr();

  // Then a short delay
  delay(1000);

  // Select a pattern to run
  int index = selectPattern(); 

  // Calculate future time to switch pattern
  timeOut = millis() + (1000 * PATTERN_DISPLAY_DURATION_SECONDS);

  // Enable time outs
  timeOutEnabled = true;

  // Start up the selected pattern
  (*patternFunctions[index])();

  // This one used for testing only
//  (*patternFunctions[0])();
}

/*******************************************************************/
/***         Display and Font Definitions and Variables          ***/
/*******************************************************************/

#define WIDTH_LANDSCAPE  320
#define MIN_X_LANDSCAPE    0
#define MAX_X_LANDSCAPE  319
#define XMID (WIDTH_LANDSCAPE / 2)
#define HEIGHT_LANDSCAPE 240
#define MIN_Y_LANDSCAPE    0
#define MAX_Y_LANDSCAPE  239
#define YMID (HEIGHT_LANDSCAPE / 2)

#define BIG_FONT_WIDTH    16
#define BIG_FONT_HEIGHT   16

/*******************************************************************/
/***           Color Constants, Variables and Functions          ***/
/*******************************************************************/

#define NUM_OF_COLOR_VALUES 256
#define MIN_COLOR_VALUE       0  
#define MAX_COLOR_VALUE     255

struct RGB {
  uint8_t red;
  uint8_t green;
  uint8_t blue;
};

// Color space conversion
// Input arguments
// hue in degrees (0 - 360.0)
// saturation in percent (0.0 - 1.0)
// value in percent (0.0 - 1.0)
// Output arguments
// red, green blue (0.0 - 1.0)
void _HSVtoRGB(float hue, float saturation, float value, float *red, float *green, float *blue) {

  int i;
  float f, p, q, t;
  if (saturation == 0) {
    // achromatic (grey)
    *red = *green = *blue = value;
    return;
  }
  hue /= 60;			// sector 0 to 5
  i = floor(hue);
  f = hue - i;			// factorial part of h
  p = value * (1 - saturation);
  q = value * (1 - saturation * f);
  t = value * (1 - saturation * (1 - f));
  switch(i) {
  case 0:
    *red = value;
    *green = t;
    *blue = p;
    break;
  case 1:
    *red = q;
    *green = value;
    *blue = p;
    break;
  case 2:
    *red = p;
    *green = value;
    *blue = t;
    break;
  case 3:
    *red = p;
    *green = q;
    *blue = value;
    break;
  case 4:
    *red = t;
    *green = p;
    *blue = value;
    break;
  default:
    *red = value;
    *green = p;
    *blue = q;
    break;
  }
}

// Create a HSV color and return it to an RGB structure
void createColor(float hue, float saturation, float value, struct RGB *color) {
  float r, g, b;

  _HSVtoRGB(hue, saturation, value, &r, &g, &b);
  (*color).red   = (uint8_t) (r * MAX_COLOR_VALUE);
  (*color).green = (uint8_t) (g * MAX_COLOR_VALUE);
  (*color).blue =  (uint8_t) (b * MAX_COLOR_VALUE);
}

// Create an HSV color
void createHSVColor(int divisions, int index, float saturation, float value, struct RGB *color) {

  float hueAngle = (360.0 * index) / divisions;
  createColor(hueAngle, saturation, value, color);
}

// Create a fully saturated HSV color
void createHSVColor(int divisions, int index, struct RGB *color) {
  index %= divisions;
  createHSVColor(divisions, index, 1.0, 1.0, color);
}

/*******************************************************************/
/***             Palette definitions and Functions               ***/
/*******************************************************************/

#define PALETTE_SIZE      256
#define NUM_OF_PALETTES     7
#define GRAYSCALE_PALETTE   0
#define SPECTRUM_PALETTE    1
#define SIN1_PALETTE        2
#define SIN2_PALETTE        3
#define SIN3_PALETTE        4
#define SIN4_PALETTE        5
#define RANDOM_PALETTE      6

// Create a palette structure for holding color information
struct RGB palette[PALETTE_SIZE];

// Generate a palette based upon parameter
void generatePalette(int paletteNumber) {

  // Create some factors for randomizing the generated palettes
  // This helps keep the display colors interesting
  float f1 = random(16, 128);
  float f2 = random(16, 128);
  float f3 = random(16, 128);

  switch(paletteNumber) {
  case GRAYSCALE_PALETTE:
    {
      // Grayscale palette
      // Sometimes the light colors at low index; other times at high index
      boolean direction = (random(2) == 1);
      if (direction) {
        for (int i = 0; i < PALETTE_SIZE; i++) {
          palette[i].red = palette[i].green = palette[i].blue = i;
        }
      }  
      else  {
        for (int i = 0; i < PALETTE_SIZE; i++) {
          palette[i].red = palette[i].green = palette[i].blue = 255 - i;
        }
      }
    }
    break;

  case SPECTRUM_PALETTE:
    {
      // Full spectrum palette at full saturation and value
      for (int i = 0; i < PALETTE_SIZE; i++) {
        createHSVColor(PALETTE_SIZE, i, &palette[i]);
      }
    }
    break;

  case SIN1_PALETTE:
    {
      // Use sin function to generate palette
      for(int i = 0; i < PALETTE_SIZE; i++) {
        palette[i].red   = 128.0 + 128 * sin(PI * i / f1);
        palette[i].green = 128.0 + 128 * sin(PI * i / f2);
        palette[i].blue  = 128.0 + 128 * sin(PI * i / f3);
      } 
    }
    break;

  case SIN2_PALETTE:
    {
      // Use sin function to generate palette - no blue
      for(int i = 0; i < 256; i++) {
        palette[i].red   = 128.0 + 128 * sin(PI * i / f1);
        palette[i].green = 128.0 + 128 * sin(PI * i / f2);
        palette[i].blue  = 0;
      }
    } 
    break;

  case SIN3_PALETTE:
    {
      // Use sin function to generate palette - no green
      for(int i = 0; i < 256; i++) {
        palette[i].red   = 128.0 + 128 * sin(PI * i / f1);
        palette[i].green = 0;
        palette[i].blue  = 128.0 + 128 * sin(PI * i / f2);
      } 
    }
    break;

  case SIN4_PALETTE:
    {
      // Use sin function to generate palette - no red
      for(int i = 0; i < PALETTE_SIZE; i++) {
        palette[i].red   = 0;
        palette[i].green = 128.0 + 128 * sin(PI * i / f1);
        palette[i].blue  = 128.0 + 128 * sin(PI * i / f2);
      } 
    }
    break;

  case RANDOM_PALETTE:
    {
      // Choose random color components
      for(int i = 0; i < PALETTE_SIZE; i++) {
        palette[i].red   = random(NUM_OF_COLOR_VALUES);
        palette[i].green = random(NUM_OF_COLOR_VALUES);
        palette[i].blue  = random(NUM_OF_COLOR_VALUES);
      } 
    }
    break;
  }
}

/*******************************************************************/
/***             Mandelbrot and Julia Set Functions              ***/
/*******************************************************************/

// The following should probably be 256 but this lengthens mandelbrot generation 
// time substantially
#define MANDELBROT_STEPS 128

int pointInMandelbrotSet(float z0Real, float z0Imag, float cReal, float cImag) {

  float x = z0Real;
  float y = z0Imag;
  float nx, ny;
  for (int i = 0; i < MANDELBROT_STEPS; i++) {

    // Calculate the real part of the sequence
    nx = (x * x) - (y * y) + cReal;
    // Calculate the imaginary part of the sequence
    ny = 2 * x * y + cImag;
    // Check magnitude at each step
    // We check if it's greater than 2
    // which means the point diverges towards infinity
    if (((nx * nx) + (ny * ny)) > 4) {
      return i;
    }
    x = nx;
    y = ny;
  }
  // Point in mandelbrot set
  return MANDELBROT_STEPS;
}

// Draw Mandelbrot set
// Drawing occurs on black background unless paintBackground is true
void drawMandelbrotSet(int paletteNumber, boolean paintBackground, float scale) {

  // Generate specified palette
  generatePalette(paletteNumber);

  if (paintBackground) {
    int colorIndex = random(PALETTE_SIZE);
    lcd.fillScr(palette[colorIndex].red, palette[colorIndex].green, palette[colorIndex].blue);
  }

  float xOfs = WIDTH_LANDSCAPE  / 2.0;
  float yOfs = HEIGHT_LANDSCAPE / 2.0;

  for (int x = 0; x < WIDTH_LANDSCAPE; x++) {
    float newReal = scale * ((float) (x - xOfs) / xOfs);

    for (int y = 0; y < HEIGHT_LANDSCAPE; y++) {
      float newImag = scale * ((float) (y - yOfs) / yOfs);

      int iterations = pointInMandelbrotSet(0, 0, newReal, newImag);
      if (iterations != MANDELBROT_STEPS) {
        // Map 128 iterations into 256 colors
        iterations *= 2;

        // Retrieve color from current palette
        lcd.setColor(palette[iterations].red, palette[iterations].green, palette[iterations].blue);
        lcd.drawPixel(x, y);
      }
    }
  }
}

// Generate a random signed 10 digit floating point value
float generateRandomFloat() {

  char str[12];

  memset(str, 0, sizeof(str));
  int index = 0;

  boolean isPositive = (random(2) == 1);
  if (isPositive) {
    str[index] = ' ';
  }  
  else  {
    str[index] = '-';
  }
  index++;
  str[index++] = '.';

  for (int i = 0; i < 9; i++) {
    int offset = random(10);
    str[index++] = '0' + offset;
  }
  return atof(str); 
}

// Draw julia set
void drawJuliaSet(int paletteNumber, float cReal, float cImag) {

  // Generate specified palette
  generatePalette(paletteNumber);

  float xOfs = WIDTH_LANDSCAPE  / 2.0;
  float yOfs = HEIGHT_LANDSCAPE / 2.0;

  for (int x = 0; x < WIDTH_LANDSCAPE; x++) {
    float newReal = (((float) x - xOfs) / xOfs);

    for (int y = 0; y < HEIGHT_LANDSCAPE; y++) {
      float newImag = (((float) y - yOfs) / yOfs);

      int iterations = pointInMandelbrotSet(newReal, newImag, cReal, cImag);
      if (iterations != MANDELBROT_STEPS) {
        // Map 128 iterations into 256 colors
        iterations *= 2;

        // Retrieve color from current palette
        lcd.setColor(palette[iterations].red, palette[iterations].green, palette[iterations].blue);
        lcd.drawPixel(x, y);
      }
    }
  }
}

/*******************************************************************/
/***                     Spirograph Functions                    ***/
/*******************************************************************/

// Greatest common divisor using Euclid's method
int gcd(int x, int y) {

  if (x % y == 0) {
    return y;
  }
  return gcd(y, x % y);
}

// Compute revolutions to complete spirograph pattern
// computed from radaii of fixed and moving circles
int revolutions(int R, int r) {

  return (int) (r / gcd(R, r)) * 7;
}

void drawSpirograph() {

  // Pick parameters randomly

  // Pick moving circle position: inside or outside
  boolean mcInside = (random(2) == 1);

  // Pick fcRadius and mcRadius accordingly
  int fcRadius, mcRadius;

  if (mcInside) {
    // Moving circle inside fixed circle
    fcRadius = random(100, HEIGHT_LANDSCAPE / 2);

    // Moving circle radius must be smaller than fixed circle radius
    mcRadius = random(46, fcRadius);
  }  
  else  {
    // Moving circle outside fixed circle
    fcRadius = random(40, 65);
    mcRadius = random(26, fcRadius);

    while ((120 + fcRadius + (2 * mcRadius)) >= HEIGHT_LANDSCAPE) {
      fcRadius = random(40, 65);
      mcRadius = random(26, fcRadius);
    }
  }

  // Pick moving circle offset
  int mcOffset = random(10, mcRadius);

  // Pick color mode: single, multi or random
  int colorMode = random(3);

  // Pick drawing mode: point or line
  boolean drawingModePoint = (random(2) == 1);

  // Calculate iterations to perform
  int iterations = revolutions(fcRadius, mcRadius);

  int redBits   = random(NUM_OF_COLOR_VALUES);
  int greenBits = random(NUM_OF_COLOR_VALUES);
  int blueBits  = random(NUM_OF_COLOR_VALUES);

  int x = 0;
  int y = 0;
  int prevx;
  int prevy;

  // Controls the step size; low values of r (e.g. 2) make jagged pattern, so s will be small
  // for low value of r; the effect is not directly proportinal, but sqrt (arbitrary)
  int exp = random(0, 5);
  int scale = pow(10, exp);
  float scaleFactor = 4.0 / scale;
  float s = scaleFactor * sqrt(mcRadius);

  int xOffset = WIDTH_LANDSCAPE  / 2;
  int yOffset = HEIGHT_LANDSCAPE / 2;

  float rSum  = fcRadius + mcRadius;
  float rDiff = fcRadius - mcRadius;

  // For multiColor, step to change for each color 
  int redStep = 1;
  int greenStep = 1;
  int blueStep = 1;

  for (float t = 0; t <= iterations; t += s) {

    prevx = x;
    prevy = y;

    // Determine color mode ?
    switch (colorMode) {
    case 0:
      // Single color mode. Use previous selected color every iteration
      break;

    case 1:
      {
        // Multi color mode
        // Randomly change one color in each iteration; makes more interesting patterns
        int whichColor = random(3); 

        switch (whichColor) {
        case 0:
          {
            if (redBits < 1) {
              redStep = 1;
            }
            if (redBits > 254) {
              redStep = -1;
            }
            redBits += redStep;
          }
          break;

        case 1:
          {
            if (greenBits < 1) {
              greenStep = 1;
            }
            if (greenBits > 254) {
              greenStep = -1;
            }
            greenBits += greenStep;
          }
          break;

        case 2:
          {
            if (blueBits < 1) {
              blueStep = 1;
            }
            if (blueBits > 254) {
              blueStep = -1;
            }
            blueBits += blueStep;
            break;
          }
        }
        break;
      }

    case 2:
      {
        // Random color mode 
        redBits   = random(NUM_OF_COLOR_VALUES);
        greenBits = random(NUM_OF_COLOR_VALUES);
        blueBits  = random(NUM_OF_COLOR_VALUES);
      }
      break;
    }
    // Select new drawing color
    lcd.setColor(redBits, greenBits, blueBits);

    if (mcInside) {
      // Moving circle is inside the fixed circle
      float exprResult = (rDiff * t) / (float) mcRadius;

      x = (int) ((rDiff * cos(t)) + (mcOffset * cos(exprResult)) + xOffset);
      y = (int) ((rDiff * sin(t)) - (mcOffset * sin(exprResult)) + yOffset);

    }  
    else  {

      // Moving circle is outside the fixed circle
      float exprResult = (rSum * t) / (float) mcRadius;
      x = (int) ((rSum * cos(t)) - (mcOffset * cos(exprResult)) + xOffset);
      y = (int) ((rSum * sin(t)) - (mcOffset * sin(exprResult)) + yOffset);
    }
    // Do drawing
    if (drawingModePoint) {
      // Drawing individual points
      lcd.drawPixel(x, y);
    }  
    else  {
      // Drawing lines
      if (t > 0) {
        lcd.drawLine(prevx, prevy, x, y);
      }
    }
  }
}

/*******************************************************************/
/***                       Plasma Functions                      ***/
/*******************************************************************/

#define NUM_OF_PLASMAS  4
#define PLASMA_TYPE_0   0
#define PLASMA_TYPE_1   1
#define PLASMA_TYPE_2   2
#define PLASMA_TYPE_3   3

void drawPlasma(int plasmaType, int paletteNumber) {

  int colorIndex;
  float value = 0;

  // Generate specified palette
  generatePalette(paletteNumber);

  // Generate some float factors to alter plasma
  float f1 = (float) random(2, 64);
  float f2 = (float) random(2, 64);
  float f3 = (float) random(2, 64);

  for (int x = 0; x < WIDTH_LANDSCAPE; x++) {
    for (int y = 0; y < HEIGHT_LANDSCAPE; y++) {

      // Determine plasma type
      switch (plasmaType) {

      case PLASMA_TYPE_0:
        {
          value = sin(sqrt((x - XMID) * (x - XMID) + (y - YMID) * (y - YMID)) / f1);
        }
        break;

      case PLASMA_TYPE_1:
        {
          value = (sin(x / f1) + sin(y / f1)) / 2.0;
        }
        break;

      case PLASMA_TYPE_2:
        {
          value = (sin(x / f1) + sin(y / f2) + sin((x + y) / f3)) / 3.0;
        }
        break;

      case PLASMA_TYPE_3:
        {
          value  = sin(x / f1);
          value += sin(y / f2);
          value += sin(sqrt(((x - XMID) * (x - XMID)) + ((y - YMID) * (y - YMID))) / f3);
          value /= 3.0;
        }
        break;
      }
      // Scale -1 ... +1 values to 0 ... 255
      value = (value * 128.0) + 128.0;
      colorIndex = ((int) value) % 256;
      lcd.setColor(palette[colorIndex].red, palette[colorIndex].green, palette[colorIndex].blue);
      lcd.drawPixel(x, y);
    }
  }
}

/*******************************************************************/
/***                   Display Pattern Functions                 ***/
/*******************************************************************/

void circlesPattern() {
  int x, y, radius;

  while (true) {
    // Draw some random circles
    for (int i= 0; i < 70; i++) {

      lcd.setColor(random(NUM_OF_COLOR_VALUES), random(NUM_OF_COLOR_VALUES), random(NUM_OF_COLOR_VALUES));

      radius = random(50) + 2;

      // Make sure to keep the circle's coordinates in bounds
      x = random(MAX_X_LANDSCAPE);
      while (((x - radius) < 1) || ((x + radius) > MAX_X_LANDSCAPE - 1)) {
        x = random(MAX_X_LANDSCAPE);
      }

      y = random(MAX_Y_LANDSCAPE);
      while (((y - radius) < 1) || ((y + radius) > MAX_Y_LANDSCAPE - 1)) {
        y = random(MAX_Y_LANDSCAPE);
      }
      // Draw some open and some filled circles
      boolean flag = (random(3) == 1);
      if (! flag) {
        lcd.drawCircle(x, y, radius);
      }  
      else  {
        lcd.fillCircle(x, y, radius);
      }
      delay(200);
    }
    checkForTimeout();
    delay(1000 * BETWEEN_PATTERN_DELAY_SECONDS);
    lcd.clrScr();
  }
}

void linesPattern() {

  int x, y, xPrev, yPrev;

  // Pick a random palette
  int paletteNumber = random(NUM_OF_PALETTES);
  generatePalette(paletteNumber);

  // Pick a random place to start within the palette
  int colorIndex = random(PALETTE_SIZE);

  xPrev = WIDTH_LANDSCAPE  / 2;
  yPrev = HEIGHT_LANDSCAPE / 2;

  while (true) {

    for (int i = 0; i < 80; i++) {
      lcd.setColor(palette[colorIndex].red, palette[colorIndex].green, palette[colorIndex].blue);
      colorIndex++; 
      colorIndex %= PALETTE_SIZE;
      x = random(MAX_X_LANDSCAPE);
      y = random(MAX_Y_LANDSCAPE);
      lcd.drawLine(x, y, xPrev, yPrev);
      xPrev = x;
      yPrev = y;
      delay(60);
    }
    checkForTimeout();
    delay(1000 * BETWEEN_PATTERN_DELAY_SECONDS);
    lcd.clrScr();
  }
}

void webPattern() {

#define ITERATIONS 16

  struct RGB color;
  int colorIndex = 0;

  while (true) {

    createHSVColor(ITERATIONS, colorIndex++, &color);
    lcd.setColor (color.red, color.green, color.blue);

    for (int y = 0; y < 240; y += 5) {
      lcd.drawLine(0, y, y * 1.33, 239);
    }    

    createHSVColor(ITERATIONS, colorIndex++, &color);
    lcd.setColor (color.red, color.green, color.blue);

    for (int y = 239; y >= 0; y -= 5) {
      lcd.drawLine(319, y, y * 1.33, 0);
    }

    createHSVColor(ITERATIONS, colorIndex++, &color);
    lcd.setColor (color.red, color.green, color.blue);

    for (int y = 239; y >= 0; y -= 5) {
      lcd.drawLine(0, y, 320 - (y * 1.33), 0);
    }

    createHSVColor(ITERATIONS, colorIndex++, &color);
    lcd.setColor (color.red, color.green, color.blue);

    for (int y = 0; y < 240; y += 5) {
      lcd.drawLine(319, y, 320 - (y * 1.33), 239);
    }

    checkForTimeout();
  }
}

void spirographPattern() {

  while (true) {

    drawSpirograph();

    checkForTimeout();
    delay(1000 * BETWEEN_PATTERN_DELAY_SECONDS);
    lcd.clrScr();
  }
}

void mandelbrotPattern() {

  float scale = 1.5;

  // Pick a random scale factor
  int scaleFactor = random(5);
  switch (scaleFactor) {
  case 0:
    scale = 1.5;
    break;
  case 1:
    scale = 1.25;
    break;
  case 2:
    scale = 1.1;
    break;
  case 3:
    scale = 2.0;
    break;
  case 4:
    scale = 2.5;
    break;
  }
  // Determine if mandelbrot set members are colored or black
  boolean paintBackground = (random(2) == 1);

  // Draw the set with specified scale and palette
  drawMandelbrotSet(random(NUM_OF_PALETTES), paintBackground, scale);

  // Force timeout after generation
  timeOut = 0;

  while (true) {
    delay(1000);
    checkForTimeout();
  }
}

void juliaPattern() {

  // Generate a complex constant as seed for Julia set
  float cReal = generateRandomFloat();
  float cImag = generateRandomFloat();

  // Make sure this complex constant diverges at (0, 0)
  // This prevents some not so interesting Julia sets from being (slowly) rendered
  while (pointInMandelbrotSet(0, 0, cReal, cImag) == MANDELBROT_STEPS) {
    cReal = generateRandomFloat();
    cImag = generateRandomFloat();
  }

  // Draw the hopefully interesting Julia set
  drawJuliaSet(random(NUM_OF_PALETTES), cReal, cImag);

  // Force timeout after generation
  timeOut = 0;

  while (true) {
    delay(1000);
    checkForTimeout();
  }
}

void bigPixelPattern() {

#define BORDER_WIDTH 16
#define BPWidth      60
#define BPHeight     40

  struct RGB color;

  while (true) {
    int numberOfColors = random(NUM_OF_COLOR_VALUES);
    int colorIndex = random(numberOfColors);

    for (int countHoriz = 0; countHoriz < 4; countHoriz++) {
      for (int countVert = 0; countVert < 4; countVert++) {
        // Calculate color
        createHSVColor(numberOfColors, colorIndex++, &color);
        lcd.setColor(color.red, color.green, color.blue);

        int xPos = BORDER_WIDTH + countHoriz * (BPWidth + BORDER_WIDTH);
        int yPos = BORDER_WIDTH + countVert * (BPHeight + BORDER_WIDTH);

        // Create filled rect
        lcd.fillRect(xPos, yPos, xPos + BPWidth + 1, yPos + BPHeight + 1);
        delay(20);
      }
    }
    // Check for termination
    checkForTimeout();
  }
}

void bigPixel2Pattern() {

  // Fit three rows of four circles onto the display
#define HBW    16
#define VBW    15
#define RADIUS 30

  while (true) {

    for (int row = 0; row < 3; row++) {
      for (int col = 0; col < 4; col++) {

        // Create random color for the circle
        lcd.setColor(random(NUM_OF_COLOR_VALUES), random(NUM_OF_COLOR_VALUES), random(NUM_OF_COLOR_VALUES));

        // Calculate location of circle
        int x = (col + 1) * (HBW + RADIUS) + (col * RADIUS);
        int y = (row + 1) * (VBW + RADIUS) + (row * RADIUS);

        // Draw the filled circle
        lcd.drawCircle(x, y, RADIUS);
      }
    }
    // Check for termination
    checkForTimeout();
  }
}

// Support routine for pattern below
void drawCircleOfCircles(int colorIndex, int numberOfCircles, int largeRadius, int smallRadius) {

  struct RGB color;

  int cx = WIDTH_LANDSCAPE  / 2;
  int cy = HEIGHT_LANDSCAPE / 2;

  for (int i = 1; i <= numberOfCircles; ++i) {
    // Calculate color
    createHSVColor(numberOfCircles, colorIndex++, &color);
    lcd.setColor(color.red, color.green, color.blue);

    float angle = i * 2 * PI / numberOfCircles;
    float x = cx + cos(angle) * largeRadius;
    float y = cy + sin(angle) * largeRadius;
    lcd.fillCircle(x, y, smallRadius);
  }
}

void circleOfCirclesPattern() {

  int width  = WIDTH_LANDSCAPE;
  float scaleFactor;

  int numberOfCircles;

  int colorIndex = 0;
  while (true) {

    for (int i = 0; i < 3; i++) {

      numberOfCircles = 8 + (i * 8);
      scaleFactor = 0.21 + (i * 0.21);

      float largeRadius = (width / 2) * scaleFactor;
      float largeCircle = 2 * PI * largeRadius;
      float smallRadius = (largeCircle / numberOfCircles) / 2;

      drawCircleOfCircles(colorIndex, numberOfCircles, largeRadius, smallRadius);
      colorIndex += 2;

      // Check for termination
      checkForTimeout();
    }
  }
}

void plasmaPattern() {

  while (true) {
    int plasmaType = random(NUM_OF_PLASMAS);
    int paletteNumber = random(NUM_OF_PALETTES);

    // Draw the specified plasma with the specified palette
    drawPlasma(plasmaType, paletteNumber);

    // Check for timeout
    checkForTimeout();
    delay(1000 * BETWEEN_PATTERN_DELAY_SECONDS);
    lcd.clrScr();
  }
}

void concentricSquaresPattern() {

  int colorIndex = random(PALETTE_SIZE);

  int x1, x2, y1, y2;

  int xInc = 3;
  int yInc = 2;

  int paletteNumber = random(NUM_OF_PALETTES);
  generatePalette(paletteNumber);

  while (true) {
    // Set initial coordinates
    x1 = XMID - 3;
    x2 = XMID + 3;
    y1 = YMID - 1;
    y2 = YMID + 1;

    while ((x1 >= 0) && (x2 < WIDTH_LANDSCAPE) &&
      (y1 >= 0) && (y2 < HEIGHT_LANDSCAPE)) {

      lcd.setColor(palette[colorIndex].red, palette[colorIndex].green, palette[colorIndex].blue); 
      colorIndex++;
      colorIndex %= PALETTE_SIZE;

      lcd.drawRect(x1, y1, x2, y2);
      x1 -= xInc;
      x2 += xInc;
      y1 -= yInc;
      y2 += yInc;

      delay(50);
    }
    // Check for timeout
    checkForTimeout();
  }
}

void concentricCirclesPattern() {

  int radius;
  int radiusInc = 1;

  // Pick a random palette
  int paletteNumber = random(NUM_OF_PALETTES);
  generatePalette(paletteNumber);

  // Pick a random place to start within the palette
  int colorIndex = random(PALETTE_SIZE);

  while (true) {

    radius = 2;

    while ((radius < XMID) && (radius < YMID)) {

      lcd.setColor(palette[colorIndex].red, palette[colorIndex].green, palette[colorIndex].blue); 
      colorIndex++;
      colorIndex %= PALETTE_SIZE;

      lcd.drawCircle(XMID, YMID, radius);
      radius += radiusInc;

      delay(50);
    }
    // Check for timeout
    checkForTimeout();
  }
}

// Support routine for display pattern below
void drawRotatingText(int radius, char *string) {

  int x, y;

  for (int degrees = 0; degrees < 360; degrees += 45) {

    float rads = degrees * PI / 180.0;

    x = XMID - radius * cos(rads);
    y = YMID - radius * sin(rads);

    if (degrees <= 90) {
      x += 8;
      y -= 8;
    }

    lcd.setColor(0, 0, 0);
    lcd.print(string, x, y, degrees);
    if (degrees == 0) {
      delay(500);
    } 
    else  {
      delay(300);
    }

    lcd.setColor(255, 255, 255);
    lcd.print(string, x, y, degrees);
    if (degrees == 0) {
      delay(300);
    } 
    else  {
      delay(200);
    }
  }
}

// Display revolving text messages
void textPattern() {

  char *str1 = "Nuts and Volts";
  int str1Length = strlen(str1);
  int str1PixelWidth = str1Length * 16;
  int radius1 = str1PixelWidth / 2;

  char *str2 = "ContemplatoR";
  int str2Length = strlen(str2);
  int str2PixelWidth = str2Length * 16;
  int radius2 = str2PixelWidth / 2;

  // Set fill and background color to white
  lcd.fillScr(255, 255, 255);
  lcd.setBackColor(255, 255, 255);

  while (true) {
    drawRotatingText(radius1, str1);

    checkForTimeout();

    drawRotatingText(radius2, str2);

    checkForTimeout();
  }
}

void vertPaletteLinesPattern() {

  while (true) {
    // Pick a random palette
    int paletteNumber = random(NUM_OF_PALETTES);
    generatePalette(paletteNumber);

    // Pick a random place to start within the palette
    int colorIndex = random(PALETTE_SIZE);
    for (int y = 0; y < HEIGHT_LANDSCAPE; y++) {

      lcd.setColor(palette[colorIndex].red, palette[colorIndex].green, palette[colorIndex].blue); 
      colorIndex++;
      colorIndex %= PALETTE_SIZE;

      lcd.drawLine(MIN_X_LANDSCAPE, y, MAX_X_LANDSCAPE, y);
      delay(50);
    }
    checkForTimeout();
    delay(1000 * BETWEEN_PATTERN_DELAY_SECONDS);
    lcd.clrScr();
  }
}

void horizPaletteLinesPattern() {

  while (true) {
    // Pick a random palette
    int paletteNumber = random(NUM_OF_PALETTES);
    generatePalette(paletteNumber);

    // Pick a random place to start within the palette
    int colorIndex = random(PALETTE_SIZE);
    for (int x = 0; x < WIDTH_LANDSCAPE; x++) {

      lcd.setColor(palette[colorIndex].red, palette[colorIndex].green, palette[colorIndex].blue); 
      colorIndex++;
      colorIndex %= PALETTE_SIZE;

      lcd.drawLine(x, MIN_Y_LANDSCAPE, x, MAX_Y_LANDSCAPE);
      delay(50);
    }
    checkForTimeout();
    delay(1000 * BETWEEN_PATTERN_DELAY_SECONDS);
    lcd.clrScr();
  }
}

void burstPattern() {

  // Randomly toggle burst background between black and gray
  if (random(2) == 1) {
    // Set background to gray
    lcd.fillScr(128, 128, 128);
  }

  // Pick the attributes of the burst
  int smallRadius = random(8, 30);
  int mediumRadius = random(smallRadius + 10, 75);
  int largeRadius = YMID;

  int degreeInc = 15;

  // Pick the burst spoke pitch
  switch(random(7)) {
  case 0:
    degreeInc = 6;
    break;
  case 1:
    degreeInc = 10;
    break;
  case 2:
    degreeInc = 12;
    break;
  case 3:
    degreeInc = 15;
    break;
  case 4:
    degreeInc = 18;
    break;
  case 5:
    degreeInc = 20;
    break;
  case 6:
    degreeInc = 30;
    break;
  }

  int xi, xo, yi, yo;
  double rads;
  struct RGB color;

  int numberOfSpokes = 360 / degreeInc;
  int colorIndex = 0;
  boolean inner = false;

  // Pick the delay for this time through
  int delayTime = random(10, 100);

  while (true) {

    for (int degree = 0; degree < 360; degree += degreeInc) {

      // Convert degrees to radians
      rads = ((double) degree)  * PI / 180.0;

      xi = smallRadius * cos(rads) + XMID;
      yi = smallRadius * sin(rads) + YMID;

      if (inner) {
        // Drawing shorter spokes
        xo = mediumRadius * cos(rads) + XMID;
        yo = mediumRadius * sin(rads) + YMID;
      }  
      else  {
        // Drawing longer spokes
        xo = largeRadius * cos(rads) + XMID;
        yo = largeRadius * sin(rads) + YMID;
      }
      // Toggle inner to outer and vise versa
      inner = ! inner;

      // Create longer spoke color
      createHSVColor(numberOfSpokes, colorIndex++, &color);

      // Set spoke color
      lcd.setColor(color.red, color.green, color.blue);

      // Draw the spoke
      lcd.drawLine(xi, yi, xo, yo);

      delay(delayTime);
    }
    colorIndex += 2;
    checkForTimeout();
  }
}

void offsetCirclePattern() {

  struct RGB color;
  boolean even = (random(2) == 0);

  while (true) {

    int numberOfColors = random(NUM_OF_COLOR_VALUES);
    int colorIndex = random(numberOfColors);
    int radiusIncrement = random(4, 14);
    int delayTime = random(20, 100);

    for (int radius = 1; radius < YMID; radius += radiusIncrement) {

      // Create and set color
      createHSVColor(numberOfColors, colorIndex++, &color);
      lcd.setColor(color.red, color.green, color.blue);

      if (even) {
        lcd.drawCircle(radius , YMID, radius);
      }  
      else  {
        lcd.drawCircle(radius, radius, radius);
      }
      delay(delayTime);

      // Create and set color
      createHSVColor(numberOfColors, colorIndex++, &color);
      lcd.setColor(color.red, color.green, color.blue);
      if (even) {
        lcd.drawCircle(WIDTH_LANDSCAPE - radius , YMID, radius);
      }  
      else  {
        lcd.drawCircle(WIDTH_LANDSCAPE - radius, radius, radius);
      }
      delay(delayTime);

      // Create and set color
      createHSVColor(numberOfColors, colorIndex++, &color);
      lcd.setColor(color.red, color.green, color.blue);
      if (even) {
        lcd.drawCircle(XMID , radius, radius);
      }  
      else  {
        lcd.drawCircle(radius, HEIGHT_LANDSCAPE - radius - 1, radius);
      }
      delay(delayTime);

      // Create and set color
      createHSVColor(numberOfColors, colorIndex++, &color);
      lcd.setColor(color.red, color.green, color.blue);
      if (even) {
        lcd.drawCircle(XMID , HEIGHT_LANDSCAPE - radius - 1, radius);
      }  
      else  {
        lcd.drawCircle(WIDTH_LANDSCAPE - radius, HEIGHT_LANDSCAPE - radius - 1, radius);
      }
      delay(delayTime);
    }
    even = ! even;

    // Check for timeout
    checkForTimeout();
    delay(1000 * BETWEEN_PATTERN_DELAY_SECONDS);
    lcd.clrScr();
  }
}









